@@ -22,6 +22,14 @@ pub enum Color {
     Magenta,
     Cyan,
     White,
+    BrightBlack,
+    BrightRed,
+    BrightGreen,
+    BrightYellow,
+    BrightBlue,
+    BrightMagenta,
+    BrightCyan,
+    BrightWhite,
     Fixed(u8),
     RGB(u8, u8, u8),
 }
@@ -41,6 +49,17 @@ impl Color {
             Color::Magenta => ansi_term::Color::Purple,
             Color::Cyan => ansi_term::Color::Cyan,
             Color::White => ansi_term::Color::White,
+
+            // Below items are a rough translations to 256 colors as
+            // we do not have bright varients available on ansi-term
+            Color::BrightBlack => ansi_term::Color::Fixed(8),
+            Color::BrightRed => ansi_term::Color::Fixed(9),
+            Color::BrightGreen => ansi_term::Color::Fixed(10),
+            Color::BrightYellow => ansi_term::Color::Fixed(11),
+            Color::BrightBlue => ansi_term::Color::Fixed(12),
+            Color::BrightMagenta => ansi_term::Color::Fixed(13),
+            Color::BrightCyan => ansi_term::Color::Fixed(14),
+            Color::BrightWhite => ansi_term::Color::Fixed(15),
         }
     }
 
@@ -55,13 +74,21 @@ impl Color {
             },
             Color::Fixed(n) => crossterm::style::Color::AnsiValue(*n),
             Color::Black => crossterm::style::Color::Black,
-            Color::Red => crossterm::style::Color::Red,
-            Color::Green => crossterm::style::Color::Green,
-            Color::Yellow => crossterm::style::Color::Yellow,
-            Color::Blue => crossterm::style::Color::Blue,
-            Color::Magenta => crossterm::style::Color::Magenta,
-            Color::Cyan => crossterm::style::Color::Cyan,
-            Color::White => crossterm::style::Color::White,
+            Color::Red => crossterm::style::Color::DarkRed,
+            Color::Green => crossterm::style::Color::DarkGreen,
+            Color::Yellow => crossterm::style::Color::DarkYellow,
+            Color::Blue => crossterm::style::Color::DarkBlue,
+            Color::Magenta => crossterm::style::Color::DarkMagenta,
+            Color::Cyan => crossterm::style::Color::DarkCyan,
+            Color::White => crossterm::style::Color::Grey,
+            Color::BrightBlack => crossterm::style::Color::DarkGrey,
+            Color::BrightRed => crossterm::style::Color::Red,
+            Color::BrightGreen => crossterm::style::Color::Green,
+            Color::BrightYellow => crossterm::style::Color::Yellow,
+            Color::BrightBlue => crossterm::style::Color::Blue,
+            Color::BrightMagenta => crossterm::style::Color::Magenta,
+            Color::BrightCyan => crossterm::style::Color::Cyan,
+            Color::BrightWhite => crossterm::style::Color::White,
         }
     }
 }
@@ -185,6 +212,7 @@ pub struct Style {
     pub foreground: Option<Color>,
     pub background: Option<Color>,
     pub font_style: FontStyle,
+    pub underline: Option<Color>,
 }
 
 impl Style {
@@ -196,12 +224,13 @@ impl Style {
 
         let mut parts: VecDeque<u8> = code
             .split(';')
-            .map(|c| u8::from_str_radix(c, 10).ok())
+            .map(|c| c.parse::<u8>().ok())
             .collect::<Option<_>>()?;
 
         let mut font_style = FontStyle::default();
         let mut foreground = None;
         let mut background = None;
+        let mut underline = None;
 
         loop {
             match parts.pop_front() {
@@ -284,6 +313,35 @@ impl Style {
                     }
                 },
                 Some(49) => background = None,
+                Some(58) => match (parts.pop_front(), parts.pop_front()) {
+                    (Some(5), Some(color)) => underline = Some(Color::Fixed(color)),
+                    (Some(2), Some(red)) => match (parts.pop_front(), parts.pop_front()) {
+                        (Some(green), Some(blue)) => underline = Some(Color::RGB(red, green, blue)),
+                        _ => {
+                            break;
+                        }
+                    },
+                    _ => {
+                        break;
+                    }
+                },
+                Some(59) => underline = None,
+                Some(90) => foreground = Some(Color::BrightBlack),
+                Some(91) => foreground = Some(Color::BrightRed),
+                Some(92) => foreground = Some(Color::BrightGreen),
+                Some(93) => foreground = Some(Color::BrightYellow),
+                Some(94) => foreground = Some(Color::BrightBlue),
+                Some(95) => foreground = Some(Color::BrightMagenta),
+                Some(96) => foreground = Some(Color::BrightCyan),
+                Some(97) => foreground = Some(Color::BrightWhite),
+                Some(100) => background = Some(Color::BrightBlack),
+                Some(101) => background = Some(Color::BrightRed),
+                Some(102) => background = Some(Color::BrightGreen),
+                Some(103) => background = Some(Color::BrightYellow),
+                Some(104) => background = Some(Color::BrightBlue),
+                Some(105) => background = Some(Color::BrightMagenta),
+                Some(106) => background = Some(Color::BrightCyan),
+                Some(107) => background = Some(Color::BrightWhite),
                 Some(_) => {
                     continue;
                 }
@@ -297,27 +355,25 @@ impl Style {
             foreground,
             background,
             font_style,
+            underline,
         })
     }
 
     /// Convert to a `ansi_term::Style` (if the `ansi_term` feature is enabled).
     #[cfg(feature = "ansi_term")]
     pub fn to_ansi_term_style(&self) -> ansi_term::Style {
-        let mut ansi_style = ansi_term::Style::default();
-
-        ansi_style.foreground = self.foreground.as_ref().map(Color::to_ansi_term_color);
-        ansi_style.background = self.background.as_ref().map(Color::to_ansi_term_color);
-
-        ansi_style.is_bold = self.font_style.bold;
-        ansi_style.is_dimmed = self.font_style.dimmed;
-        ansi_style.is_italic = self.font_style.italic;
-        ansi_style.is_underline = self.font_style.underline;
-        ansi_style.is_blink = self.font_style.rapid_blink || self.font_style.slow_blink;
-        ansi_style.is_reverse = self.font_style.reverse;
-        ansi_style.is_hidden = self.font_style.hidden;
-        ansi_style.is_strikethrough = self.font_style.strikethrough;
-
-        ansi_style
+        ansi_term::Style {
+            foreground: self.foreground.as_ref().map(Color::to_ansi_term_color),
+            background: self.background.as_ref().map(Color::to_ansi_term_color),
+            is_bold: self.font_style.bold,
+            is_dimmed: self.font_style.dimmed,
+            is_italic: self.font_style.italic,
+            is_underline: self.font_style.underline,
+            is_blink: self.font_style.rapid_blink || self.font_style.slow_blink,
+            is_reverse: self.font_style.reverse,
+            is_hidden: self.font_style.hidden,
+            is_strikethrough: self.font_style.strikethrough,
+        }
     }
 
     /// Convert to a `crossterm::style::ContentStyle` (if the `crossterm` feature is enabled).
@@ -327,6 +383,7 @@ impl Style {
             foreground_color: self.foreground.as_ref().map(Color::to_crossterm_color),
             background_color: self.background.as_ref().map(Color::to_crossterm_color),
             attributes: self.font_style.to_crossterm_attributes(),
+            underline_color: self.underline.as_ref().map(Color::to_crossterm_color),
         }
     }
 }
@@ -339,22 +396,39 @@ mod tests {
         code: &str,
         foreground: Option<Color>,
         background: Option<Color>,
+        underline: Option<Color>,
         font_style: FontStyle,
     ) {
         let style = Style::from_ansi_sequence(code).unwrap();
         assert_eq!(foreground, style.foreground);
         assert_eq!(background, style.background);
+        assert_eq!(underline, style.underline);
         assert_eq!(font_style, style.font_style);
     }
 
     #[test]
     fn parse_simple() {
-        assert_style("31", Some(Color::Red), None, FontStyle::default());
-        assert_style("47", None, Some(Color::White), FontStyle::default());
+        assert_style("31", Some(Color::Red), None, None, FontStyle::default());
+        assert_style("47", None, Some(Color::White), None, FontStyle::default());
+        assert_style(
+            "91",
+            Some(Color::BrightRed),
+            None,
+            None,
+            FontStyle::default(),
+        );
+        assert_style(
+            "107",
+            None,
+            Some(Color::BrightWhite),
+            None,
+            FontStyle::default(),
+        );
         assert_style(
             "32;40",
             Some(Color::Green),
             Some(Color::Black),
+            None,
             FontStyle::default(),
         );
     }
@@ -368,16 +442,22 @@ mod tests {
 
     #[test]
     fn parse_font_style() {
-        assert_style("00;31", Some(Color::Red), None, FontStyle::default());
-        assert_style("03;34", Some(Color::Blue), None, FontStyle::italic());
-        assert_style("06;34", Some(Color::Blue), None, FontStyle::rapid_blink());
-        assert_style("01;36", Some(Color::Cyan), None, FontStyle::bold());
+        assert_style("00;31", Some(Color::Red), None, None, FontStyle::default());
+        assert_style("03;34", Some(Color::Blue), None, None, FontStyle::italic());
+        assert_style(
+            "06;34",
+            Some(Color::Blue),
+            None,
+            None,
+            FontStyle::rapid_blink(),
+        );
+        assert_style("01;36", Some(Color::Cyan), None, None, FontStyle::bold());
         let italic_and_bold = FontStyle {
             bold: true,
             italic: true,
             ..Default::default()
         };
-        assert_style("01;03", None, None, italic_and_bold);
+        assert_style("01;03", None, None, None, italic_and_bold);
     }
 
     #[test]
@@ -388,15 +468,21 @@ mod tests {
 
     #[test]
     fn support_reset_of_styles() {
-        assert_style("01;31", Some(Color::Red), None, FontStyle::bold());
-        assert_style("01;31;22", Some(Color::Red), None, FontStyle::default());
+        assert_style("01;31", Some(Color::Red), None, None, FontStyle::bold());
+        assert_style(
+            "01;31;22",
+            Some(Color::Red),
+            None,
+            None,
+            FontStyle::default(),
+        );
     }
 
     #[test]
     fn parse_font_style_backwards() {
-        assert_style("34;03", Some(Color::Blue), None, FontStyle::italic());
-        assert_style("36;01", Some(Color::Cyan), None, FontStyle::bold());
-        assert_style("31;00", Some(Color::Red), None, FontStyle::default());
+        assert_style("34;03", Some(Color::Blue), None, None, FontStyle::italic());
+        assert_style("36;01", Some(Color::Cyan), None, None, FontStyle::bold());
+        assert_style("31;00", Some(Color::Red), None, None, FontStyle::default());
     }
 
     #[test]
@@ -405,24 +491,49 @@ mod tests {
             "38;5;115",
             Some(Color::Fixed(115)),
             None,
+            None,
             FontStyle::default(),
         );
         assert_style(
             "00;38;5;115",
             Some(Color::Fixed(115)),
             None,
+            None,
             FontStyle::default(),
         );
         assert_style(
             "01;38;5;119",
             Some(Color::Fixed(119)),
             None,
+            None,
             FontStyle::bold(),
         );
         assert_style(
             "38;5;119;01",
             Some(Color::Fixed(119)),
             None,
+            None,
+            FontStyle::bold(),
+        );
+        assert_style(
+            "58;5;115",
+            None,
+            None,
+            Some(Color::Fixed(115)),
+            FontStyle::default(),
+        );
+        assert_style(
+            "00;58;5;115",
+            None,
+            None,
+            Some(Color::Fixed(115)),
+            FontStyle::default(),
+        );
+        assert_style(
+            "01;58;5;119",
+            None,
+            None,
+            Some(Color::Fixed(119)),
             FontStyle::bold(),
         );
     }
@@ -433,18 +544,28 @@ mod tests {
             "38;2;115;3;100",
             Some(Color::RGB(115, 3, 100)),
             None,
+            None,
             FontStyle::default(),
         );
         assert_style(
             "38;2;115;3;100;3",
             Some(Color::RGB(115, 3, 100)),
             None,
+            None,
             FontStyle::italic(),
         );
         assert_style(
             "48;2;100;200;0;1;38;2;0;10;20",
             Some(Color::RGB(0, 10, 20)),
             Some(Color::RGB(100, 200, 0)),
+            None,
+            FontStyle::bold(),
+        );
+        assert_style(
+            "48;2;100;200;0;1;38;2;0;10;20;58;2;64;64;64",
+            Some(Color::RGB(0, 10, 20)),
+            Some(Color::RGB(100, 200, 0)),
+            Some(Color::RGB(64, 64, 64)),
             FontStyle::bold(),
         );
     }
